Package: jmvReadWrite
Title: Read and Write 'jamovi' Files ('.omv')
Version: 0.4.14
Author: Sebastian Jentschke [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-2576-5432>)
Authors@R: 
    person(given = "Sebastian",
           family = "Jentschke",
           role = c("aut", "cre", "cph"),
           email = "sebastian.jentschke@uib.no",
           comment = c(ORCID = "0000-0003-2576-5432"))
Maintainer: Sebastian Jentschke <sebastian.jentschke@uib.no>
Description: The free and open a statistical spreadsheet 'jamovi'
    (<https://www.jamovi.org>) aims to make statistical analyses easy and
    intuitive. 'jamovi' produces syntax that can directly be used in R (in
    connection with the R-package 'jmv'). Having import / export routines for
    the data files 'jamovi' produces ('.omv') permits an easy transfer of
    data and analyses between 'jamovi' and R.
License: AGPL-3
LazyData: true
Language: en-GB
Encoding: UTF-8
VignetteBuilder: knitr
URL: https://sjentsch.github.io/jmvReadWrite/
BugReports: https://github.com/sjentsch/jmvReadWrite/issues
Depends: R (>= 3.5.0)
Imports: jsonlite, methods, zip
Suggests: jmv, jmvcore (>= 2.4.7), foreign, haven, knitr, rmarkdown,
        RProtoBuf, testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Config/roxygen2/version: 8.0.0
Packaged: 2026-07-13 14:48:26 UTC; sje025
Repository: CRAN
Date/Publication: 2026-07-13 15:10:02 UTC
Built: R 4.6.1; ; 2026-07-13 19:00:59 UTC; unix
