# Generated by roxygen2: do not edit by hand

export(aggregate_omv)
export(arrange_cols_omv)
export(combine_cols_omv)
export(convert_to_omv)
export(describe_omv)
export(distances_omv)
export(label_vars_omv)
export(long2wide_omv)
export(merge_cols_omv)
export(merge_rows_omv)
export(read_omv)
export(replace_omv)
export(search_omv)
export(sort_omv)
export(transform_vars_omv)
export(transpose_omv)
export(wide2long_omv)
export(write_omv)
