% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ToothGrowth}
\alias{ToothGrowth}
\title{The Effect of Vitamin C on Tooth Growth in Guinea Pigs}
\format{
A data frame with 60 rows and 6 variables:
\describe{
\item{\code{ID}}{ID of the guinea pig}
\item{\code{supp}}{Supplement type (VC: Vitamin C or OJ: Orange juice)}
\item{\code{supp2}}{Transformation of the supplement type (factor to numerical: VC = 1; OJ = 2)}
\item{\code{dose}}{Dose in grams / day}
\item{\code{dose2}}{Dose in grams / day}
\item{\code{len}}{Tooth length}
\item{\code{logLen}}{Natural logarithm of the tooth length (len)}
}
}
\usage{
ToothGrowth
}
\description{
The Effect of Vitamin C on Tooth Growth in Guinea Pigs
}
\keyword{datasets}
